
/*
 * $Id$
 *
 * Revision History
 * ================
 * $Log$
 * ===========
 *
 *
 * RF12.c
 *
 *  Created on: 29-Oct-2009
 *      Author: Clive Webster
 */
#include "RF12.h"

typedef enum {
	BAND_315 = 0,
	BAND_433 = 1,
	BAND_868 = 2,
	BAND_915 = 3
} BAND;

#define SET_BAND(rf12,band) spi16(rf12, 0x80c7 | (band<<4 ) )

// Send a 16 bit word and return the received 16 bit word
static uint16_t spi16(HOPE_RF12* rf12, uint16_t out){
	uint16_t rtn = 0;

	// select the device
	spiDeviceSelect(rf12, TRUE);

	// Transmit
	rtn = _spiSendWord(__spiDeviceGetBus(rf12->_device_.bus),out);

	// De-select the device
	spiDeviceSelect(rf12, FALSE);
	return rtn;
}

// Initialise the RF12 on power up
static void rf12_init(HOPE_RF12* rf12){

//	spi16(rf12, 0x80d7);	// Either: 433 Hz band
//	spi16(rf12, 0x80e7);	// Or:     868 Hz band
	SET_BAND(rf12, BAND_868);

	spi16(rf12, 0x82d9);	// Power management
	spi16(rf12, 0xa67c);	// Nudge the frequency in the band 868.3 MHz, or 434.15 MHz
	spi16(rf12, 0xc647);	// Data rate command: Set transmit/receive bit rate 4.8kbps
	spi16(rf12, 0x94a0);	// Receiver control command:  VDI,FAST,134kHz,0dBm,-103dBm
	spi16(rf12, 0xc2ac);	// Data filter command: AL,!ml,DIGITAL FILTER,DQD4
	spi16(rf12, 0xca81);	// FIFO and reset command: FIFO8,SYNC,!ff,DR
	spi16(rf12, 0xc483);	// AFC command: @PWR,NO RSTRIC,!st,!fi,OE,EN

	spi16(rf12, 0x9854);	// TX Configuration control command: -12db
//	spi16(rf12, 0x9850);	// TX Configuration control command: 0db - ie full power

	spi16(rf12, 0xe000);	// Wake-up timer command: disable
	spi16(rf12, 0xc800);	// Low duty cycle command: disable
	spi16(rf12, 0xc000);	// Low battery command: disable


}
