/*
 * $Id: ADXL335.c,v 1.6 2010/01/24 17:02:29 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: ADXL335.c,v $
 * Revision 1.6  2010/01/24 17:02:29  clivewebster
 * *** empty log message ***
 *
 * Revision 1.5  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.4  2009/11/02 18:16:23  clivewebster
 * Added revision log
 *
 * ===========
 *
 *  Created on: 09-Sep-2009
 *      Author: Clive Webster
 *  Based on original code by Admin at www.societyofrobots.com
 *
 *  Assumes 3v3 supply where 0.330v represents one 'g'
 *
 */
#include "ADXL335.h"
#include "../../../a2d.h"
#include "../../../fraction.h"

// 14.811 at 10 bit
static const uint16_t PROGMEM adxl335_frac[]     = {14,  2,  4, 17, 473, 0};

static ACCEL_TYPE __read_channel(ADC_CHANNEL pin){
	uint16_t adc = a2dConvert10bit(pin);
	ACCEL_TYPE rtn = fraction32(adc, adxl335_frac);
	return rtn - 5006;
}

// Read all the values and store into the device
static void __ad_adxl335_read(SENSOR* sensor){
	ADXL335* accel = (ADXL335*)sensor;

	__accelerometerSetX(&accel->accelerometer,__read_channel(accel->x_pin));
	__accelerometerSetY(&accel->accelerometer,__read_channel(accel->y_pin));
	__accelerometerSetZ(&accel->accelerometer,__read_channel(accel->z_pin));
}

// Datasheet: Turn-on time is dependent on CX, CY, CZ and is approximately 160  CX or CY or CZ + 1 ms,
// 			   where CX, CY, CZ are in microfarads (uF).
// My board has 0.1uF caps so 160*0.1 + 1 => 17ms
// 20ms between readings since these caps give 50Hz readings ie every 20ms
ACCELEROMETER_CLASS c_ADXL335 = MAKE_ACCELEROMETER_CLASS(null,&__ad_adxl335_read,20,20);



