
/*
 * $Id: MMA7260QT.c,v 1.3 2010/01/24 17:10:16 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: MMA7260QT.c,v $
 * Revision 1.3  2010/01/24 17:10:16  clivewebster
 * Use ADC channels in constructor rather than IOPins
 *
 * Revision 1.2  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.1  2009/11/01 00:50:25  clivewebster
 * *** empty log message ***
 *
 * ===========
 *
 *
 * MMA7260QT.c
 *
 *  Created on: 29-Oct-2009
 *      Author: Clive Webster
 */
#include "MMA7260QT.h"

#include "../../../a2d.h"
#include "../../../fraction.h"

static const uint16_t PROGMEM frac_1_5g[] = {3,  5, 0};		// times 3.5
static const uint16_t PROGMEM frac_2g[]   = {4,  2, 7, 43, 0};	// times 4.6666
static const uint16_t PROGMEM frac_4g[]   = {9,  3, 0};		// times 9.333333
static const uint16_t PROGMEM frac_6g[]   = {14,  0};	    // times 14

static ACCEL_TYPE __read_channel(ADC_CHANNEL pin, uint8_t mode){
	const uint16_t * frac;
	switch(mode){
	case 1:
		frac = frac_2g;
		break;
	case 2:
		frac = frac_4g;
		break;
	case 3:
		frac = frac_6g;
		break;
	default:
		frac = frac_1_5g;
	}
	uint16_t adc = a2dConvert10bit(pin) - 511;
	return fraction32(adc,frac);
}

// Read all the values and store into the device
static void read(SENSOR* sensor){
	FS_MMA7260QT* accel = (FS_MMA7260QT*)sensor;
	__accelerometerSetX(&accel->accelerometer,__read_channel(accel->x_pin, accel->mode));
	__accelerometerSetY(&accel->accelerometer,__read_channel(accel->y_pin, accel->mode));
	__accelerometerSetZ(&accel->accelerometer,__read_channel(accel->z_pin, accel->mode));

}

ACCELEROMETER_CLASS c_FS_MMA7260QT = MAKE_ACCELEROMETER_CLASS(null,&read,2,0);

