/*
 * $Id: HMC6343.c,v 1.4 2010/02/04 19:52:14 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: HMC6343.c,v $
 * Revision 1.4  2010/02/04 19:52:14  clivewebster
 * Added functions to set the refresh rate
 *
 * Revision 1.3  2009/11/02 18:20:31  clivewebster
 * Added revision log
 *
 * ===========
 * HMC6343.c
 *
 *  Created on: 21-Sep-2009
 *      Author: Clive Webster
 */
#include "HMC6343.h"
#include "../../../i2c_master.h"
#include "../../../timer.h"

#define CMD_GET_HEADING 0x50
#define CMD_WRITE_EEPROM 0xf1
typedef struct s_cmd_50{
	uint8_t heading_msb;		// 0
	uint8_t heading_lsb;		// 1
	uint8_t pitch_msb;			// 2
	uint8_t pitch_lsb;			// 3
	uint8_t roll_msb;			// 4
	uint8_t roll_lsb;			// 5
} HEADING_REPLY;

// Set a 1Hz refresh period
void HMC6343_1Hz(HMC6343* compass){
	uint8_t cmd[] = {CMD_WRITE_EEPROM, 5, 0};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}

// Set a 5Hz refresh period (default)
void HMC6343_5Hz(HMC6343* compass){
	uint8_t cmd[] = {CMD_WRITE_EEPROM, 5, 1};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}

// Set a 10Hz refresh period
void HMC6343_10Hz(HMC6343* compass){
	uint8_t cmd[] = {CMD_WRITE_EEPROM, 5, 2};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}

static void __HMC6343_read(SENSOR* sensor){
	HMC6343* compass = (HMC6343*)sensor;
	uint8_t  cmd = CMD_GET_HEADING;			// post heading and pitch
	HEADING_REPLY reply;

	// Post heading data
	i2cMasterSend(compass->i2cAddress,1,&cmd);

	// Datasheet says to wait for 1ms before reading response
	delay_ms(1);

	// receive the response
	i2cMasterReceive(compass->i2cAddress, sizeof(HEADING_REPLY), (uint8_t*)(&reply));

	// Get heading in 10ths of a degree 0->3600 and round to degrees
	uint16_t heading10ths = reply.heading_msb;
	heading10ths <<= 8;
	heading10ths |= reply.heading_lsb;
	compass->compass.bearingDegrees = (heading10ths + 5)/10;

	// Get roll +- 900 tenths of a degree
	int16_t roll10ths = reply.roll_msb;
	roll10ths <<= 8;
	roll10ths |= reply.roll_lsb;
	compass->compass.rollDegrees = (roll10ths + 5)/10;

	// Get pitch +- 900 tenths of a degree
	int16_t pitch10ths = reply.pitch_msb;
	pitch10ths <<= 8;
	pitch10ths |= reply.pitch_lsb;
	compass->compass.pitchDegrees = (pitch10ths + 5)/10;

}

// Requires a 500ms delay before being read for the first time
// The fastest it can go is 10Hz ie every 100ms
COMPASS_CLASS c_HMC6343 = MAKE_COMPASS_CLASS(null,&__HMC6343_read, 500 , 100 );
