
/*
 * $Id: HMC6352.c,v 1.3 2010/02/04 22:59:38 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: HMC6352.c,v $
 * Revision 1.3  2010/02/04 22:59:38  clivewebster
 * Fixed init problem - cast of mismatched pointer
 *
 * Revision 1.2  2010/02/04 19:57:12  clivewebster
 * Add functions to change the refresh rate
 *
 * Revision 1.1  2010/01/30 21:21:18  clivewebster
 * Added in Version 1.14
 *
 * ===========
 *
 *
 * HMC6352.c
 *
 *  Created on: 29-Jan-2010
 *      Author: Clive Webster
 */
#include "HMC6352.h"
#include "../../../i2c_master.h"
#include "../../../timer.h"

#define CMD_GET_HEADING 0x41
#define CMD_WRITE_RAM   0x47

typedef struct s_cmd_50{
	uint8_t heading_msb;		// 0
	uint8_t heading_lsb;		// 1
} HEADING_REPLY;

void HMC6352_1Hz(HMC6352* compass){
	// Put into continuous mode 1Hz
	uint8_t cmd[] = {CMD_WRITE_RAM, 0x74, 0b00010010};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}

void HMC6352_5Hz(HMC6352* compass){
	// Put into continuous mode 5Hz
	uint8_t cmd[] = {CMD_WRITE_RAM, 0x74, 0b00110010};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}
void HMC6352_10Hz(HMC6352* compass){
	// Put into continuous mode 10Hz
	uint8_t cmd[] = {CMD_WRITE_RAM, 0x74, 0b01010010};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}
void HMC6352_20Hz(HMC6352* compass){
	// Put into continuous mode 20Hz
	uint8_t cmd[] = {CMD_WRITE_RAM, 0x74, 0b01110010};

	// Post heading data
	i2cMasterSend(compass->i2cAddress,sizeof(cmd),cmd);
}

static void __HMC6352_init(SENSOR* sensor){
	HMC6352* compass = (HMC6352*)sensor;
	HMC6352_20Hz(compass);
}

static void __HMC6352_read(SENSOR* sensor){
	HMC6352* compass = (HMC6352*)sensor;
	HEADING_REPLY reply;

	// Post heading data
// Not need in continuous mode
//	uint8_t  cmd = CMD_GET_HEADING;			// post heading and pitch
//	i2cMasterSend(compass->i2cAddress,1,&cmd);
//	delay_ms(6);							// Datasheet says to wait for 6ms before reading response

	// receive the response
	i2cMasterReceive(compass->i2cAddress, sizeof(HEADING_REPLY), (uint8_t*)(&reply));

	// Get heading in 10ths of a degree 0->3600 and round to degrees
	uint16_t heading10ths = reply.heading_msb;
	heading10ths <<= 8;
	heading10ths |= reply.heading_lsb;
	compass->compass.bearingDegrees = (heading10ths + 5)/10;

}

// Requires a 500ms delay before being read for the first time
// The fastest it can go is 20Hz ie every 50ms
COMPASS_CLASS c_HMC6352 = MAKE_COMPASS_CLASS(&__HMC6352_init,&__HMC6352_read, 500 , 50 );

