/*
 * $Id
 *
 * Revision History
 * ================
 * $Log
 *
 * ===========
 * 04_Sonar.c
 *
 *  Created on: 16-Nov-2009
 *      Author: Clive Webster
 *
 *  Sound travels at 1130ft/sec or 13,560in/sec or 73.7463uS per inch
 *
 *  Since we are detecting the echo then the sound needs to travel out and back so it will take twice
 *  as long ie 147.4926254uS per inch or 58.06796274uS per cm.
 *
 *  We use the clock to measure the pulse length in uS and then divide by 58.06796274 to get cm.
 *  This is the same as multiplying by 0.0172212 which as a fraction is 1/59 + 1/3676
 *
 *
 */
#include "SRF04_Sonar.h"
#include "../../../fraction.h"

// 0.0172212
static const uint16_t PROGMEM srf04_frac[]     = {0, 59, 3676, 0};


// Read all the values and store into the device
static void __srf04_read(SENSOR* sensor){
	TICK_COUNT duration;

	Devantech_SRF04* device = (Devantech_SRF04*)sensor;

	// initialise the pins
	pin_make_output(device->out);
	pin_make_input(device->in,FALSE);

	// 10us trigger pulse
	pin_pulseOut(device->out,10);

	// Measure the inbound pulse
	duration = pin_pulseIn(device->in);

	device->distance.cm = fraction32(duration, srf04_frac);
}

// Requires a 50ms delay between readings
DISTANCE_CLASS c_Devantech_SRF04 = MAKE_DISTANCE_CLASS(null,&__srf04_read,0,50);

