
/*
 * $Id: GP2Y3A002K0F.c,v 1.1 2010/01/30 21:21:49 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: GP2Y3A002K0F.c,v $
 * Revision 1.1  2010/01/30 21:21:49  clivewebster
 * Added new sensors
 *
 * ===========
 *
 *
 * GP2Y3A002K0F.c
 *
 *  Created on: 30-Jan-2010
 *      Author: Clive Webster
 *
 *  A single unit with 5 distance sensors built in. Each sensor led is selected and then
 *  read using the single ADC output
 *
 *  On selecting an LED it takes between 16.5ms and 20.2ms to take the first reading and
 *  a further 5ms for it to appear on the output pin
 *
 *  You need to pull Vin low for at least 5ms between LEDs
 *
 */

#include "GP2.h"
// Requires the floating point library
#include <math.h>

#include "../../../timer.h"

// Numer of ms to wait until taking reading (20.2 + 5) = 25.2ms minimum
#define _DELAY_MS_ 26

static void readNext(Sharp_GP2Y3A002K0F* device){
	// Set vin high
//	pin_high(device->vin);

	// Enable the LED
	pin_high(device->led[device->pinNo]);

	// Start measuring time interval from now
	device->distance.sensor.lastRead = clockGetus();
}

// Iinitialse the device
static void __GP2Y3A002K0F_init(SENSOR* sensor){
	Sharp_GP2Y3A002K0F* device = (Sharp_GP2Y3A002K0F*)sensor;
	// Make all leds into output pins and set low
	for(uint8_t i=0; i<5; i++){
		pin_make_output(device->led[i]);
		pin_low(device->led[i]);
	}
	// Make vin an output pin and activate
	pin_make_output(device->vin);
	pin_high(device->vin);

	// Start reading the first LED
	device->pinNo = 0;
	readNext(device);
}

// Called when _DELAY_MS has passed
static void __GP2Y3A002K0F_read(SENSOR* sensor){
	Sharp_GP2Y3A002K0F* device = (Sharp_GP2Y3A002K0F*)sensor;

	// Read the current value
	uint16_t adc = a2dConvert10bit(device->adcPin);
	DISTANCE_TYPE cm = 19022.0 * pow(adc, -1.119);
	device->led_cm[device->pinNo] = cm;

	// Turn everything off
	pin_low(device->led[device->pinNo]);
//	pin_low(device->vin);

	// Select the next led
	if(device->pinNo == 4){
		// Reset back to first
		device->pinNo = 0;

		// Calc the average distance
		double total = 0;
		for(uint8_t i=0; i<5; i++ ){
			total += device->led_cm[i];
		}
		total += 2.5;
		total /= 5;
		device->distance.cm = (DISTANCE_TYPE)total;

	}else{
		// Next led
		device->pinNo += 1;
	}

	// Start reading the next pin
	readNext(device);

}

DISTANCE_CLASS c_Sharp_GP2Y3A002K0F = MAKE_DISTANCE_CLASS( \
		&__GP2Y3A002K0F_init, 	/* init routine */  \
		&__GP2Y3A002K0F_read,	/* read routine */  \
		0,						/* startp delay ms */ \
		_DELAY_MS_);			/* delay between readings */

