/*
 * $Id: quadrature.c,v 1.2 2009/11/02 18:36:19 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: quadrature.c,v $
 * Revision 1.2  2009/11/02 18:36:19  clivewebster
 * Added revision log
 *
 * ===========
 * quadrature.c
 *
 *  Created on: 03-Apr-2009
 *      Author: Clive Webster
 */

#include "quadrature.h"
#include "../../../pinChange.h"


// Called when channel A has changed
static void __encChA(const IOPin* io,boolean val, volatile void* data){
	QUADRATURE* encoder = (QUADRATURE*)data;
	boolean chB = pin_get(encoder->channelB);
	if(val ^ chB){
		encoder->counter++;
	}else{
		encoder->counter--;
	}
}

// Initialise the device
void __quadrature_init(SENSOR* sensor){
	QUADRATURE* encoder = (QUADRATURE*)sensor;
	pin_make_input(encoder->channelA, FALSE);
	pin_make_input(encoder->channelB, FALSE);
	pin_change_attach(encoder->channelA, &__encChA, encoder);
}


void __quadrature_read(SENSOR* sensor){
	ENCODER_TYPE ticks;
	QUADRATURE* encoder = (QUADRATURE*)sensor;

	// Disable interrupts whilst reading counter.
	CRITICAL_SECTION_START;
	ticks = encoder->counter;
	CRITICAL_SECTION_END;

	if(encoder->inverted){
		ticks *= -1;
	}

	// Store the last read value
	encoder->encoder.value=ticks;
}

ENCODER_CLASS c_quadrature = MAKE_ENCODER_CLASS(&__quadrature_init,&__quadrature_read,0,0);

void quadratureSubtract(QUADRATURE* encoder, ENCODER_TYPE count){
	if(encoder->inverted){
		count*=-1;
	}
	CRITICAL_SECTION_START;
	encoder->counter -= count;
	CRITICAL_SECTION_END;
}

// Return how many ticks there are per revolution
uint16_t quadratureTicksPerRevolution(const QUADRATURE* encoder){
	uint16_t rtn = encoder->numStripes;
	return rtn * 2;
}
