/*
 * $Id: quadrature.h,v 1.4 2009/12/11 17:17:57 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: quadrature.h,v $
 * Revision 1.4  2009/12/11 17:17:57  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.3  2009/11/02 18:36:19  clivewebster
 * Added revision log
 *
 * ===========
 * quadrature.h
 *
 *  Created on: 03-Apr-2009
 *      Author: Clive Webster
 *
 *  A generic quadrature encoder.
 *  This uses two input signals called Channel A and Channel B which are 90 degrees out of phase
 *
 *  On a rising edge of Channel A then Channel B will tell us which direction we are moving in
 *
 *  These two signals can be applied to any pin that can generate pin change interrupts. This excludes
 *  the ATMega8 and ATMega32 but the ATMega168 and ATMega640 can support this on most I/O pins.
 *  Look at the device file (in the dev folder) for your processor and look for the PCINT_PINS array.
 *  This  will contain the list of IO pins that could be used.
 *
 *
 */

#ifndef QUADRATURE_H_
#define QUADRATURE_H_

#if !defined(BUILDING_LIBRARY)
// we are building the app
 #include "../../../pinChange.h"
#endif

#include "../_encoder_common.h"

extern ENCODER_CLASS c_quadrature;

typedef struct s_quadrature{
	ENCODER		 encoder;
	const IOPin* channelA;			// The Channel A pin
	const IOPin* channelB;			// The Channel B pin
	volatile ENCODER_TYPE counter;	// The tick counter - don't access this directly !!
	const boolean inverted:1;		// Is it inverted (has the same meaning as with a motor)
	const uint8_t numStripes;		// Number of stripes on the wheel
}QUADRATURE;

#define MAKE_GENERIC_QUADRATURE(channelA, channelB, inverted, numStripes) {\
	MAKE_ENCODER_SENSOR(c_quadrature), \
	channelA, \
	channelB, \
	0, \
	inverted, \
	numStripes \
	}

// Subtract a value from the counter
void quadratureSubtract(QUADRATURE* encoder, ENCODER_TYPE count);

// Return how many ticks there are per revolution
uint16_t quadratureTicksPerRevolution(const QUADRATURE* encoder);



#endif /* QUADRATURE_H_ */
