/*
 * $Id: IDG300.c,v 1.3 2010/01/24 17:29:35 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: IDG300.c,v $
 * Revision 1.3  2010/01/24 17:29:35  clivewebster
 * Use ADC channels in constructor rather than IOPins
 *
 * Revision 1.2  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.1  2009/09/26 18:01:40  clivewebster
 * Initial version
 *
 * ===========

 *
 *  Created on: 23-Sep-2009
 *      Author: Clive Webster
 *
 */
#include "IDG300.h"
#include "../../../a2d.h"
#include "../../../fraction.h"

// 2.445 at 10 bit
static const uint16_t PROGMEM idg300_frac[]     = {2,  3,  10, 100, 0};

static GYRO_TYPE __read_channel(ADC_CHANNEL pin, int16_t error){
	uint16_t adc = a2dConvert10bit(pin);
	adc -= error;

	GYRO_TYPE rtn;
	rtn = fraction32(adc, idg300_frac);
	rtn -= 751;
	return rtn;
}

static void __idg300_init(SENSOR* sensor){
	IDG300* device = (IDG300*)sensor;
	// The default is that device should be outputing 1.5v ie ADC10 = 307
	device->x_error = a2dConvert10bit(device->x_pin) - ((uint16_t)(307));
	device->y_error = a2dConvert10bit(device->y_pin) - ((uint16_t)(307));
}

// Read all the values and store into the device
static void __idg300_read(SENSOR* sensor){
	IDG300* device = (IDG300*)sensor;
	device->gyro.x_axis_degrees_per_second = __read_channel(device->x_pin, device->x_error);
	device->gyro.y_axis_degrees_per_second = __read_channel(device->y_pin, device->y_error);
}

// 200ms startup time, 20ms between readings
GYRO_CLASS c_IDG300 = MAKE_GYRO_CLASS(&__idg300_init,&__idg300_read,200,20);



