/*
 * $Id: IDG300.h,v 1.3 2010/01/24 17:29:35 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: IDG300.h,v $
 * Revision 1.3  2010/01/24 17:29:35  clivewebster
 * Use ADC channels in constructor rather than IOPins
 *
 * Revision 1.2  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.1  2009/09/26 18:01:40  clivewebster
 * Initial version
 *
 * ===========

 *
 * 2 axis gyro
 * See: http://www.sparkfun.com/datasheets/Components/IDG-300_Datasheet.pdf
 *
 * Requires 2 ADC channels
 *
 *  Created on: 23-Sep-2009
 *      Author: Clive Webster
 *
 *
 *  Specs summary:
 *  3v supply
 *  1.5v output for zero
 *  Power on time: 200ms
 *  If slow:-
 *  Sensitivity: 2mV per degree per second
 *  Full range:  500 degrees per second
 *  Equation for ADC10 reading to result = 2.445 * adc - 750.61
 *
 */

#ifndef IDG300_H_
#define IDG300_H_
#include "../_gyro_common.h"
#include "../../../a2d.h"

// The class used to read a raw value
extern GYRO_CLASS c_IDG300;

typedef struct s_IDG300{
	GYRO 		 gyro;
	ADC_CHANNEL  x_pin;
	ADC_CHANNEL  y_pin;
	int16_t		x_error;		// The x adc value at init
	int16_t		y_error;		// The y adc value at init
} IDG300;

// Create the sensor using X, Y pins
#define MAKE_IDG300(pinX, pinY) { \
	MAKE_GYRO_SENSOR(c_IDG300), \
	pinX,\
	pinY,\
	0,0}

#endif /* IDG300_H_ */
