/*
 * $Id: IDG500.c,v 1.6 2010/01/24 17:29:35 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: IDG500.c,v $
 * Revision 1.6  2010/01/24 17:29:35  clivewebster
 * Use ADC channels in constructor rather than IOPins
 *
 * Revision 1.5  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.4  2009/11/02 18:38:42  clivewebster
 * Added revision log
 *
 * ===========
 *
 *  Created on: 23-Sep-2009
 *      Author: Clive Webster
 *
 */
#include "IDG500.h"
#include "../../../a2d.h"
#include "../../../fraction.h"

// 0.5371 at 10 bit
static const uint16_t PROGMEM idg500_slow_frac[]     = {0,  2,  27, 0};
// 2.445 at 10 bit
static const uint16_t PROGMEM idg500_fast_frac[]     = {2,  3,  10, 100, 0};

static GYRO_TYPE __read_channel(ADC_CHANNEL pin, boolean slow, int16_t error){
	uint16_t adc = a2dConvert10bit(pin);
	adc -= error;

	GYRO_TYPE rtn;
	if(slow){
		rtn = fraction32(adc, idg500_slow_frac);
		rtn -= 148;
	}else{
		rtn = fraction32(adc, idg500_fast_frac);
		rtn -= 675;
	}
	return rtn;
}

static void __idg500_init(SENSOR* sensor){
	IDG500* device = (IDG500*)sensor;
	// The default is that device should be outputing 1.35v ie ADC10 of 276
	device->x_error = a2dConvert10bit(device->x_pin) - ((uint16_t)(276));
	device->y_error = a2dConvert10bit(device->y_pin) - ((uint16_t)(276));
}

// Read all the values and store into the device
static void __idg500_read(SENSOR* sensor){
	IDG500* device = (IDG500*)sensor;
	device->gyro.x_axis_degrees_per_second = __read_channel(device->x_pin, device->slow, device->x_error);
	device->gyro.y_axis_degrees_per_second = __read_channel(device->y_pin, device->slow, device->y_error);
}

// 200ms startup time, 20ms between readings
GYRO_CLASS c_IDG500 = MAKE_GYRO_CLASS(&__idg500_init,&__idg500_read,200,20);



