
/*
 * $Id: LPR530AL.c,v 1.3 2010/01/24 17:27:40 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: LPR530AL.c,v $
 * Revision 1.3  2010/01/24 17:27:40  clivewebster
 * Use ADC channels in constructor rather than IOPins
 *
 * Revision 1.2  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.1  2009/10/01 19:01:16  clivewebster
 * Created
 *
 * ===========
 *
 *
 * LPR530AL.c
 *
 *  Created on: 01-Oct-2009
 *      Author: Clive Webster
 */

#include "LPR530AL.h"
#include "../../../a2d.h"
#include "../../../fraction.h"

// 1.4674 at 10 bit
static const uint16_t PROGMEM lpr530al_slow_frac[]     = {1, 3, 8, 112, 0};
// 5.8997 at 10 bit
static const uint16_t PROGMEM lpr530al_fast_frac[]     = {5, 2, 3,  16, 286, 0};

static GYRO_TYPE __read_channel(ADC_CHANNEL pin, boolean slow, int16_t error){
	uint16_t adc = a2dConvert10bit(pin);
	adc -= error;

	GYRO_TYPE rtn;
	if(slow){
		rtn = fraction32(adc, lpr530al_slow_frac);
		rtn -= 369;
	}else{
		rtn = fraction32(adc, lpr530al_fast_frac);
		rtn -= 1485;
	}
	return rtn;
}

static void __lpr530al_init(SENSOR* sensor){
	LPR530AL* device = (LPR530AL*)sensor;
	// The default is that device should be outputing 1.23v ie ADC10 of 251
	device->x_error = a2dConvert10bit(device->x_pin) - ((uint16_t)(251));
	device->y_error = a2dConvert10bit(device->y_pin) - ((uint16_t)(251));
}

// Read all the values and store into the device
static void __lpr530al_read(SENSOR* sensor){
	LPR530AL* device = (LPR530AL*)sensor;
	device->gyro.x_axis_degrees_per_second = __read_channel(device->x_pin, device->slow, device->x_error);
	device->gyro.y_axis_degrees_per_second = __read_channel(device->y_pin, device->slow, device->y_error);
}

// 200ms startup time
GYRO_CLASS c_LPR530AL = MAKE_GYRO_CLASS(&__lpr530al_init,&__lpr530al_read,200,0);



