
/*
 * $Id: LY530ALH.c,v 1.3 2010/01/24 17:27:39 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: LY530ALH.c,v $
 * Revision 1.3  2010/01/24 17:27:39  clivewebster
 * Use ADC channels in constructor rather than IOPins
 *
 * Revision 1.2  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.1  2009/10/01 19:01:16  clivewebster
 * Created
 *
 * ===========
 *
 *
 * LY530ALH.c
 *
 *  Created on: 01-Oct-2009
 *      Author: Clive Webster
 */

#include "LY530ALH.h"
#include "../../../a2d.h"
#include "../../../fraction.h"

// 1.4674 at 10 bit
static const uint16_t PROGMEM ly530alh_slow_frac[]     = {1, 3, 8, 112, 0};
// 5.8997 at 10 bit
static const uint16_t PROGMEM ly530alh_fast_frac[]     = {5, 2, 3,  16, 286, 0};

static GYRO_TYPE __read_channel(ADC_CHANNEL pin, boolean slow, int16_t error){
	uint16_t adc = a2dConvert10bit(pin);
	adc -= error;

	GYRO_TYPE rtn;
	if(slow){
		rtn = fraction32(adc, ly530alh_slow_frac);
		rtn -= 369;
	}else{
		rtn = fraction32(adc, ly530alh_fast_frac);
		rtn -= 1485;
	}
	return rtn;
}

static void __ly530alh_init(SENSOR* sensor){
	LY530ALH* device = (LY530ALH*)sensor;
	// The default is that device should be outputing 1.23v ie ADC10 of 251
	device->z_error = a2dConvert10bit(device->z_pin) - ((uint16_t)(251));
}

// Read all the values and store into the device
static void __ly530alh_read(SENSOR* sensor){
	LY530ALH* device = (LY530ALH*)sensor;
	device->gyro.z_axis_degrees_per_second = __read_channel(device->z_pin, device->slow, device->z_error);
}

// 200ms startup time
GYRO_CLASS c_LY530ALH = MAKE_GYRO_CLASS(&__ly530alh_init,&__ly530alh_read,200,0);



