/*
 * $Id: MPX5100a.h,v 1.4 2010/01/24 17:22:24 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: MPX5100a.h,v $
 * Revision 1.4  2010/01/24 17:22:24  clivewebster
 * *** empty log message ***
 *
 * Revision 1.3  2009/12/11 17:17:56  clivewebster
 * Fixed #include for Unix
 *
 * Revision 1.2  2009/11/02 18:41:41  clivewebster
 * Added revision log
 *
 * ===========
 *
 * Pressure sensor that can be used to measure depth or altitude
 *
 * See: http://www.datasheetcatalog.org/datasheet/motorola/MPX5100D.pdf
 *
 * Requires 1 ADC channel
 *
 *  Created on: 23-Sep-2009
 *      Author: Clive Webster
 *
 *
 *  Specs summary:
 *  5v supply
 *  Power on time: 20ms
 *  Sensitivity: 45mV per kPa
 *
 *  The returned value is relative to when the device was initialised with pressureInit
 *
 */

#ifndef MPX5100A_H_
#define MPX5100A_H_
#include "../_pressure_common.h"
#include "../../../a2d.h"

// The class used to read a raw value
extern PRESSURE_CLASS c_MPX5100A;

typedef struct s_MPX5100A{
	PRESSURE	 pressure;
	ADC_CHANNEL  adcPin;
	uint16_t	 initial;		// The adc value at init
} MPX5100A;

#define MAKE_MPX5100A(adcPin) { \
	MAKE_PRESSURE_SENSOR(c_MPX5100A), \
	adcPin,\
	0}

#endif /* MPX5100A_H_ */
