/*
 * $Id: _pressure_common.h,v 1.2 2009/11/02 18:41:41 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: _pressure_common.h,v $
 * Revision 1.2  2009/11/02 18:41:41  clivewebster
 * Added revision log
 *
 * ===========
 *  Created on: 24-Sep-2009
 *      Author: Clive Webster
 *
 *  Defines the common interface for all pressure sensors
 *  Returns a value in kPa
 *
 */

#ifndef PRESSURE_H_
#define PRESSURE_H_
#include "../_sensor_common.h"

// Define the data type used by all sensors
// Changing this will mean changing pressureDump
#define PRESSURE_TYPE int16_t

// Define forward references
struct s_pressure;

// Define the api for the class
typedef struct PROGMEM c_pressure{
	SENSOR_CLASS sensor;						// include parent class
} PRESSURE_CLASS;

#define MAKE_PRESSURE_CLASS(init,read,startup,delay) {MAKE_SENSOR_CLASS(init,read,startup,delay)}

// Define the data for each pressure sensor
typedef struct s_pressure{
	SENSOR			sensor;						// Include fields from parent type
	PRESSURE_TYPE	kPa;						// Last read value
} PRESSURE;

#define MAKE_PRESSURE_SENSOR(class) {MAKE_SENSOR(&(class.sensor)),(PRESSURE_TYPE)0}

// Create macros to make the virtual methods easier to call
#define pressureRead(device) _sensorRead_(&device.pressure.sensor)
#define pressureInit(device) _sensorInit_(&device.pressure.sensor)

// Dump the value of a sensor to the rprintf destination
void _pressureDump(const PRESSURE* device);
#define pressureDump(device) _pressureDump(&device.pressure);



#endif /* PRESSURE_H_ */

