/*
 * $Id: _spi_common.c,v 1.3 2009/11/02 18:50:27 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: _spi_common.c,v $
 * Revision 1.3  2009/11/02 18:50:27  clivewebster
 * Added revision log
 *
 * ===========
 * _spi_common.c
 *
 *  Created on: 23-Jun-2009
 *      Author: Clive Webster
 */
#include "_spi_common.h"



void _spiDeviceSelect(SPI_DEVICE* device, boolean active){
	_SPI* bus = __spiDeviceGetBus(device->bus);

	if(active){
		// make sure all other devices are made inactive
		for(int deviceNo=0;deviceNo < bus->numDevices; deviceNo++){
			SPI_DEVICE* device = (SPI_DEVICE *)pgm_read_word(&bus->devices[deviceNo]);
			_spiDeviceSelect(device,FALSE);
		}

		// Now set the mode and order for the bus for this device
		if(bus->mode!=device->mode){
			_spiSetMode(bus,device->mode);
		}
		if(bus->order!=device->order){
			_spiSetDataOrder(bus,device->order);
		}

		if(device->select!=null){
			// Make output low to select
			pin_low(device->select);
		}
	}else{
		if(device->select!=null){
			// Make output high to deselect
			pin_high(device->select);
		}
	}
}



//-------------- Implement common methonds
uint16_t _spiSendWord(_SPI* spi, uint16_t data){
	// Send MSB
	uint16_t rxData = _spiSendByte( spi, ((data>>8) & 0x00FF) )<<8;
	// send LSB
	rxData |= _spiSendByte( spi, data & 0x00FF) ;
    return rxData;
}

//--------------- Implement the virtual functions
void _spiInit(_SPI* spi, boolean master){
	if(!spi->initialised){
		const SPI_CLASS* class = spi->class;
		void (*fn)(_SPI*,boolean) = (void (*)(_SPI*, boolean))pgm_read_word(&class->init);
		for(int deviceNo=0;deviceNo < spi->numDevices; deviceNo++){
			SPI_DEVICE* device = (SPI_DEVICE *)pgm_read_word(&spi->devices[deviceNo]);
			device->bus = (struct s_spi_common*)spi;
			if(device->select!=null){
				// Make into output pin and deselect
				pin_make_output(device->select);
				pin_high(device->select);
			}
		}

		fn(spi,master);
		spi->initialised = TRUE;
	}
}
void _spiOff(_SPI* spi){
	if(spi->initialised){
		const SPI_CLASS* class = spi->class;
		void (*fn)(_SPI*) = (void (*)(_SPI*))pgm_read_word(&class->off);
		fn(spi);
		spi->initialised = FALSE;
	}
}
void _spiSetClock(_SPI* spi, SPI_CLOCK clock){
	if(spi->clock!=clock){
		spi->clock = clock;
		const SPI_CLASS* class = spi->class;
		void (*fn)(_SPI*,SPI_CLOCK) = (void (*)(_SPI*, SPI_CLOCK))pgm_read_word(&class->clock);
		fn(spi,clock);
	}
}
void _spiSetDataOrder(_SPI* spi, SPI_DATA_ORDER order){
	if(spi->order != order){
		spi->order = order;
		const SPI_CLASS* class = spi->class;
		void (*fn)(_SPI*,SPI_DATA_ORDER) = (void (*)(_SPI*, SPI_DATA_ORDER))pgm_read_word(&class->order);
		fn(spi,order);
	}
}
void _spiSetMode(_SPI* spi, SPI_MODE mode){
	if(spi->mode != mode){
		spi->mode = mode;
		const SPI_CLASS* class = spi->class;
		void (*fn)(_SPI*,SPI_MODE) = (void (*)(_SPI*, SPI_MODE))pgm_read_word(&class->mode);
		fn(spi,mode);
	}
}
uint8_t _spiSendByte(_SPI* spi, uint8_t data){
	const SPI_CLASS* class = spi->class;
	uint8_t (*fn)(_SPI*,uint8_t) = (uint8_t (*)(_SPI*, uint8_t))pgm_read_word(&class->send);
	return fn(spi,data);
}

