/*
 * $Id: buffer.c,v 1.5 2010/01/24 12:12:03 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: buffer.c,v $
 * Revision 1.5  2010/01/24 12:12:03  clivewebster
 * *** empty log message ***
 *
 * Revision 1.4  2009/11/02 18:54:01  clivewebster
 * Added revision log
 *
 * ===========
 * buffer.c
 *
 *  Created on: 15-Mar-2009
 *      Author: Clive Webster
 */
#include "buffer.h"

// initialize a buffer to start at a given address and have given size
void bufferInit(cBuffer* buffer, unsigned char *start, unsigned short size){
	// begin critical section
	CRITICAL_SECTION_START;
		// set start pointer of the buffer
		buffer->dataptr = start;
		buffer->size = size;
		// initialize index and length
		buffer->dataindex = buffer->datalength = 0;
		// end critical section
	CRITICAL_SECTION_END;
}


boolean bufferGet(cBuffer* buffer, uint8_t* rtn){
	boolean ok=FALSE;
	// begin critical section
	CRITICAL_SECTION_START;
	// make sure we have a receive buffer
	if(buffer && buffer->size){
		// make sure we have data
		if(buffer->datalength){
			// get the first character from buffer
			*rtn = buffer->dataptr[buffer->dataindex];
			// move index down and decrement length
			buffer->dataindex = (buffer->dataindex + 1) % buffer->size;
			buffer->datalength--;
			ok = TRUE;
		}
	}

	// end critical section
	CRITICAL_SECTION_END;
	return ok;
}


// add a byte to the end of the buffer
boolean	bufferPut(cBuffer* buffer, unsigned char data){
	boolean rtn=FALSE; // failure

	// begin critical section
	CRITICAL_SECTION_START;

		// make sure the buffer has room
		if(buffer->datalength < buffer->size)
		{
			// save data byte at end of buffer
			buffer->dataptr[(buffer->dataindex + buffer->datalength) % buffer->size] = data;
			// increment the length
			buffer->datalength++;
			// return success
			rtn = TRUE;
		}
		// end critical section
	CRITICAL_SECTION_END;
	return rtn;
}

boolean bufferIsFull(cBuffer* buffer){
	boolean rtn;
	CRITICAL_SECTION_START;
	rtn = (buffer->datalength == buffer->size);
	CRITICAL_SECTION_END;
	return rtn;
}

// flush (clear) the contents of the buffer
void bufferFlush(cBuffer* buffer){
	// begin critical section
	CRITICAL_SECTION_START;
		// flush contents of the buffer
		buffer->datalength = 0;
		// end critical section
	CRITICAL_SECTION_END;
}

