/*
 * $Id: buffer.h,v 1.3 2009/11/02 18:54:01 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: buffer.h,v $
 * Revision 1.3  2009/11/02 18:54:01  clivewebster
 * Added revision log
 *
 * ===========
 *
 */

#ifndef _BUFFER_H_
#define _BUFFER_H_

#include "libdefs.h"

// structure/typdefs
typedef struct struct_cBuffer{
	unsigned char *dataptr;			///< the physical memory address where the buffer is stored
	unsigned short size;			///< the allocated size of the buffer
	unsigned short datalength;		///< the length of the data currently in the buffer
	unsigned short dataindex;		///< the index into the buffer where the data starts
} cBuffer;

// Constructor
#define MAKE_BUFFER(bytes) {bytes,sizeof(bytes),0,0}


// function prototypes

//! initialize a buffer to start at a given address and have given size
void bufferInit(cBuffer* buffer, unsigned char *start, unsigned short size);
boolean bufferGet(cBuffer* buffer, uint8_t* rtn);


//! dump (discard) the first numbytes from the front of the buffer
void bufferDump(cBuffer* buffer, unsigned short numbytes);

//! get a byte at the specified index in the buffer (kind of like array access)
// ** note: this does not remove the byte that was read from the buffer
unsigned char	bufferGetAtIndex(cBuffer* buffer, unsigned short index);

//! add a byte to the end of the buffer
boolean	bufferPut(cBuffer* buffer, unsigned char data);

boolean bufferIsFull(cBuffer* buffer);

//! flush (clear) the contents of the buffer
void bufferFlush(cBuffer* buffer);

#endif
//@}
