/*
 * $Id: color2.c,v 1.5 2010/02/04 19:40:14 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: color2.c,v $
 * Revision 1.5  2010/02/04 19:40:14  clivewebster
 * Add function to compare two colors
 *
 * Revision 1.4  2009/11/02 18:54:58  clivewebster
 * *** empty log message ***
 *
 * ===========
 * color2.c
 *
 *  Created on: 02-Sep-2009
 *      Author: Clive Webster
 */
#include "color.h"
#include <string.h>		// for memcpy

// Convert the color to YUV
COLOR_YUV* color2yuv(const COLOR * src, COLOR* dest){
	switch(src->colorSpace){
	case YUV:
		// Nothing to do
		if(src != dest){
			memcpy(dest, src, sizeof(COLOR));
		}
		break;
	case RGB:{
			/*
			Y  =  16 + 1/256 * (  65.738 * R'd + 129.057 * G'd +  25.064 * B'd)
			Cb = 128 + 1/256 * (- 37.945 * R'd -  74.494 * G'd + 112.439 * B'd)
			Cr = 128 + 1/256 * ( 112.439 * R'd -  94.154 * G'd -  18.285 * B'd)
			*/

			int r = src->bands.rgb.r;
			int g = src->bands.rgb.g;
			int b = src->bands.rgb.b;


			int y =  16.0 + 1.0/256.0 * (  65.738 * r + 129.057 * g +  25.064 * b);
			int u = 128.0 + 1.0/256.0 * (- 37.945 * r -  74.494 * g + 112.439 * b);
			int v = 128.0 + 1.0/256.0 * ( 112.439 * r -  94.154 * g -  18.285 * b);

			y = CLAMP(y,0,255);
			u = CLAMP(u,0,255);
			v = CLAMP(v,0,255);

			// Convert rgb to yuv
			colorSetYUV(dest,y,u,v);
			break;
		}
	}
	return &dest->bands.yuv;

}

