/*
 * $Id: iopin.h,v 1.3 2009/11/02 18:58:48 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: iopin.h,v $
 * Revision 1.3  2009/11/02 18:58:48  clivewebster
 * Added revision log
 *
 * ===========
 *
 */
#ifndef _IOPIN_H_
#define _IOPIN_H_

#include <avr/pgmspace.h>
#include <stddef.h>
#include "errors.h"
#include "libdefs.h"

/******************************************************

	Perform I/O

*******************************************************/



/******************************************************

	Set a port pin to high

******************************************************/
void pin_high(const IOPin* io);

/******************************************************

	Set a port pin to low

******************************************************/
void pin_low(const IOPin* io);

/******************************************************

	Read the value of an I/O pin and
	return TRUE if it is high or FALSE if low

******************************************************/
boolean pin_is_high(const IOPin* io);

#define pin_get(io) pin_is_high(io)

/******************************************************

	Read the value of an I/O pin and
	return TRUE if it is low or FALSE if high,
	eg
	if(pin_is_low(B2)){
	   .... PortB pin 2 is low ....
	}

******************************************************/
#define pin_is_low(io)  !pin_is_high(io)


/******************************************************

	Set an output pin high or low
	eg
	pin_set(B2, TRUE); // To set PortB pin 2 high

******************************************************/
void pin_set(const IOPin* io, boolean val);



/******************************************************

	Toggle an output pin ie if low then make high,
	if high then make low
	eg
	pin_toggle(B2); // Toggle PortB pin 2

******************************************************/
void pin_toggle(const IOPin* io);


/******************************************************

	Set the pin as an input, specifying if you need
	a pullup resistor (TRUE/FALSE)

	ie pin_make_input(B2, TRUE); // to  make B2 an input with pullup
******************************************************/
void pin_make_input(const IOPin* io, boolean pullup);

/******************************************************

	Set the pin as an output pin

	ie pin_make_output(B2); // to  make B2 an output
******************************************************/
void pin_make_output(const IOPin* io);



/******************************************************

	Is the pin an output pin
	return TRUE if it is high or FALSE if low


******************************************************/
boolean pin_is_output(const IOPin* io);


/******************************************************

	Read the value of an I/O pin and
	return TRUE if it is low or FALSE if high,
	eg
	if(pin_is_low(B2)){
	   .... PortB pin 2 is low ....
	}

******************************************************/
#define pin_is_input(io)  !pin_is_output(io)

/******************************************************

	Output a pulse of the given duration.
	io: The pin
	us: The duration of the pulse

	This will set the pin as an output and it will
	remain as an output on return

******************************************************/
void pin_pulseOut(const IOPin* io, TICK_COUNT us);

/******************************************************

	Measure the length of an input pulse
	io: The pin
	return: The pulse duration in us

	This will set the pin as an input and it will
	remain as an input on return

******************************************************/
TICK_COUNT pin_pulseIn(const IOPin* io);

#endif
