/*
 * $Id$
 *
 * Revision History
 * ================
 * $Log$
 * ===========
 *
 *
 * pid.h
 *
 *  Created on: 01-Nov-2009
 *      Author: Clive Webster
 *
 *  A generic PID controller based on pseudo code from Wikipedia:
 *  http://en.wikipedia.org/wiki/PID_controller
 *
    previous_error = 0
	integral = 0
	start:
		error = setpoint - actual_position
		integral = integral + (error*dt)
		derivative = (error - previous_error)/dt
		output = (Kp*error) + (Ki*integral) + (Kd*derivative)
		previous_error = error
		wait(dt)
		goto start
 *
 */

#ifndef PID_H_
#define PID_H_

typedef struct s_pid {
	float	kP;				// proportional scale factor
	float	kI;				// integral scale factor
	float	kD;				// derivative scale factor
	float   integral;		// The sum of the integral component
	float	setPoint;		// The desired output position
	float	prevError;		// The previous error amount
	TICK_COUNT lastTime;	// The last time I was called by setActual
} PID;

// Update the PID with the desired target position - the setPoint
void pidSetTarget(PID* pid,float setPoint);

// Update the PID with the actual position and return the new output
float pidSetActual(PID* pid,float actual);

// Construct a new PID controller
#define MAKE_PID(kP,kI,kD) {kP,kI,kD,(float)(0.0),(float)(0.0),(float(0.0),(TICK_COUNT)0}

#endif /* PID_H_ */
