/*
 * $Id: pwm.h,v 1.1 2009/10/26 18:49:24 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: pwm.h,v $
 * Revision 1.1  2009/10/26 18:49:24  clivewebster
 * *** empty log message ***
 *
 * ===========
 *
 *
 * pwm.h
 *
 *  Created on: 22-Oct-2009
 *      Author: Clive Webster
 *
 * Provide general hardware PWM support to allow for things like dimming LEDs
 *
 * This can only be performed on the PWM output pins of the timers.
 *
 */

#ifndef PWM_H_
#define PWM_H_

#include "iopin.h"

// The duty cycle can be set between and 100
typedef uint8_t DUTY_CYCLE;

// ------------------------------------------

// Set up PWM on the given pin with the given frequency and duty cycle
// returns FALSE if errors or TRUE if set up correctly in which case actualDeciHertz,if not null,
// has the actual deciHertz.
boolean pwmInitDeciHertz(const IOPin* pin, uint32_t deciHertz, DUTY_CYCLE duty, uint32_t* actualDeciHertz);
boolean pwmInitHertz(const IOPin* pin, uint32_t hertz, DUTY_CYCLE duty, uint32_t* actualHertz);

// Turn PWM off for the given pin
void pwmOff(const IOPin* pin);

// Set the duty cycle
void pwmSetDutyCycle(const IOPin* pin, DUTY_CYCLE duty);

// Get the current duty cycle
DUTY_CYCLE pwmGetDutyCycle(const IOPin* pin);

#endif /* PWM_H_ */
