/*
 * $Id$
 *
 * Revision History
 * ================
 * $Log$
 * ===========
 * sdCard.h
 *
 *  Created on: 02-Jul-2009
 *      Author: Clive Webster

	      _________________
	     /  1 2 3 4 5 6 78 |	<- view of SD card looking at contacts
	    / 9                |	Pins 8 and 9 are present only on SD cards and not MMC cards
	    |        SD Card   |
	    |                  |
	    /\/\/\/\/\/\/\/\/\/\
 	    1 - CS   (chip select)          - wire to any available I/O pin(*)
	    2 - DIN  (data in, card<-host)  - wire to SPI MOSI pin
	    3 - VSS  (ground)               - wire to ground
	    4 - VDD  (power)     			- wire to 3.3V power
	    5 - SCLK (data clock)           - wire to SPI SCK pin
	    6 - VSS  (ground)               - wire to ground
	    7 - DOUT (data out, card->host) - wire to SPI MISO pin

	    (*) you must define this chip select I/O pin in MAKE_SD_CARD

	    The only other parameter you need for MAKE_SD_CARD is the master spi bus that the device is
	    attached to which may be either hardware/software/UART spi implementation

	     Note that this is work-in-progress. Not Complete.


 */

#ifndef SDCARD_H_
#define SDCARD_H_

// The SD card is an spi device
#include "_spi_common.h"


typedef struct s_sd_card {
	SPI_DEVICE  _device_;
	uint8_t     response;	// The last R1 response

	uint8_t*	buffer;		// The memory buffer for the card, allocated on the fly
	uint16_t	block_size; // The I/O block size
	uint32_t	num_blocks;	// The number of available blocks
	uint32_t	capacityMB; // Usable size of the card
} SD_CARD;


// Define a new SD Card. These use Mode 0 in MSB order
// Parameters:
// bus    - is the SPI bus ie hardware, software, or UART
// select - is the IOPin used to select the device
#define MAKE_SD_CARD(bus, select) {MAKE_SPI_DEVICE(SPI_MODE_0,SPI_DATA_ORDER_MSB,select),0,null,0,0,0 }

// Initialise the card
// Returns FALSE if there was a problem, or TRUE if OK
boolean sdCardReset(SD_CARD* card);


#endif /* SDCARD_H_ */
