
/*
 * $Id: servosConfig.c,v 1.1 2010/02/06 13:26:04 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: servosConfig.c,v $
 * Revision 1.1  2010/02/06 13:26:04  clivewebster
 * Allow setServoConfig to change the settings for a servo
 *
 * ===========
 *
 *
 * servosConfig.c
 *
 *  Created on: 06-Feb-2010
 *      Author: Clive Webster
 *
 * Change the config settings for a given servo
 */
#include "servos.h"

void servoSetConfig(SERVO* servo, uint16_t center, uint16_t range){
	servo->center_us = center;						// Set the new center value
	servo->range_us = range;						// Set the new range value

	SERVO_DRIVER* driver = servo->driver;			// Get the current driver
	if(driver){
		// The servo is 'live'
		CRITICAL_SECTION_START;

		// Take the useable servo range and calculate the min/max pulses
		// (SERVO_CYCLE*1000) = servo_cycle
		//        min			    x
		// x = (min * g_servo_cycle)/(SERVO_CYCLE*1000)
		uint32_t min = center - range;
		uint32_t m1 =  servo->top / 1000;
		uint32_t m2 =  m1 * min;
		uint32_t m3 =  m2 / SERVO_CYCLE;
		if( (m3 & 0xFFFF0000UL)!=0){
			// doesn't fit
			setError(SERVO_TIMING);
			m3 = 0xFFFFUL;
		}
		servo->min_ticks = m3;

		uint32_t max = center + range;
		m2 =  m1 * max;
		m3 =  m2 / SERVO_CYCLE;
		if( (m3 & 0xFFFF0000UL)!=0){
			// doesn't fit
			setError(SERVO_TIMING);
			m3 = 0xFFFFUL;
		}
		servo->max_ticks = m3;
		CRITICAL_SECTION_END;
	}
}
