/*
 * $Id: spisw.c,v 1.4 2009/11/02 19:05:46 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: spisw.c,v $
 * Revision 1.4  2009/11/02 19:05:46  clivewebster
 * Added revision log
 *
 * ===========
 * spisw.c
 *
 *  Created on: 24-Jun-2009
 *      Author: Clive Webster
 */
#include "spisw.h"
//#include "pinChange.h"
#include "timer.h"

// Define the implementations of the virtual classes
static void 	__spiSWInit(_SPI* spi, boolean master);
static void 	__spiSWOff(_SPI* spi);
static void 	__spiSWSetClock(_SPI* spi, SPI_CLOCK clock);
static void 	__spiSWSetDataOrder(_SPI* spi,SPI_DATA_ORDER order);
static void 	__spiSWSetMode(_SPI* spi,SPI_MODE mode);
static uint8_t 	__spiSWSendByte(_SPI* spi, uint8_t data);

SPI_CLASS c_sw_spi = MAKE_SPI_CLASS(&__spiSWInit, &__spiSWOff,&__spiSWSetClock,&__spiSWSetDataOrder, &__spiSWSetMode,&__spiSWSendByte );


// Called under interrupts when the chip select pin has changed
//void slaveListener(const IOPin* io,boolean val, volatile void* data){
//	if(val==FALSE){
//		// Chip IS selected - so read in the next byte
//
//	}
//}



//------------- Private methods - dont call directly -----
static void __spiSWInit(_SPI* _spi, boolean master){
	SPI_SW* spi = (SPI_SW*)_spi;

    if(master){
    	pin_make_output(spi->SCLK); 		// make clock an output
    	pin_make_output(spi->MOSI); 		// make MOSI an output
    	pin_make_input(spi->MISO,FALSE);  	// make MISO an input with no pullup
//    	if(spi->SELECT!=null){
//    		pin_make_output(spi->SELECT);	// make it an output
//    		pin_high(spi->SELECT);			// put chip select high
//    	}
    }else{
    	pin_make_input(spi->SCLK,FALSE);  	// make clock an input with no pullup
    	pin_make_input(spi->MOSI,FALSE);  	// make MOSI an input with no pullup
    	pin_make_output(spi->MISO); 		// make MISO an output
//    	pin_make_input(spi->SELECT,TRUE);  	// make select an input with a pullup
//    	pin_change_attach(spi->SELECT,&slaveListener,spi);
    }

    __spiSWSetClock(_spi,_spi->clock);
    __spiSWSetDataOrder(_spi,_spi->order);
    __spiSWSetMode(_spi,_spi->mode);

}

static void __spiSWOff(_SPI* _spi){
//	SPI_SW* spi = (SPI_SW*)_spi;
//	if(spi->SELECT!=null){
//		pin_change_dettach(spi->SELECT);
//	}
}

static void __spiSWSetClock(_SPI* _spi, SPI_CLOCK clock){}
static void __spiSWSetDataOrder(_SPI* _spi,SPI_DATA_ORDER order){}

static void __spiSWSetMode(_SPI* _spi,SPI_MODE mode){
	SPI_SW* spi = (SPI_SW*)_spi;
	// Set the clock to its initial state
	pin_set(spi->SCLK, (mode == SPI_MODE_2 || mode == SPI_MODE_3));
}


static uint8_t __spiReadMISO(const SPI_SW* spi, uint8_t data){
	if(spi->_spi_.order == SPI_DATA_ORDER_MSB){
		// MSB is sent first
		data<<=1;
		if(pin_is_high(spi->MISO)){
			data |= 1;
		}
	}else{
		// LSB is sent first
		data>>=1;
		if(pin_is_high(spi->MISO)){
			data |= 0x80;
		}
	}
	return data;
}

static uint8_t __spiWriteMOSI(const SPI_SW* spi, uint8_t data){
	if(spi->_spi_.order == SPI_DATA_ORDER_MSB){
		// MSB is sent first
		pin_set(spi->MOSI, (data & 0x80));
		data<<=1;
	}else{
		// LSB is sent first
		pin_set(spi->MOSI, (data & 1));
		data>>=1;
	}
	return data;
}

// only support modes 0 and 2
static uint8_t __spiSWSendByte(_SPI* _spi, uint8_t data){
	SPI_SW* spi = (SPI_SW*)_spi;
	uint8_t rtn=0;
	uint8_t delay = _spi->clock / 6;	// div by 2 as there are 2 delays, delay loop is approx 3 cycles

	for(int i=0; i<8; i++){

		// Write the output pin
		data = __spiWriteMOSI(spi,data);

		// Toggle the clock (high)
		pin_toggle(spi->SCLK);

		// delay
		if(delay){
			_delay_loop_1(delay);
		}

		// Toggle the clock (low)
		pin_toggle(spi->SCLK);

		// delay
		if(delay){
			_delay_loop_1(delay);
		}

		// Sample the input pin
		rtn = __spiReadMISO(spi,rtn);

	}

	return rtn;

}
