/*
 * $Id: switch.h,v 1.1 2010/01/24 18:02:22 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: switch.h,v $
 * Revision 1.1  2010/01/24 18:02:22  clivewebster
 * *** empty log message ***
 *
 * ===========
 *
 *
 * switch.h
 *
 *  Created on: 11-Dec-2009
 *      Author: Clive Webster
 *
 *  A simple class to wrap an input pin as a manual switch.
 *
 *  The switch should be connected from the input pin to ground.
 *  We turn on the internal pullup resistor so that the default input value is
 *  high and goes low when pressed. Note that contact bounce may make it alternate
 *  high/low for a short period.
 */

#ifndef SWITCH_H_
#define SWITCH_H_

#include "iopin.h"

typedef struct s_switch {
	const IOPin* pin;
} SWITCH;

#define MAKE_SWITCH(pin) {pin}

static __inline__ boolean SWITCH_pressed(SWITCH* sw){
	return pin_is_low(sw->pin);
}

static __inline__ boolean SWITCH_released(SWITCH* sw){
	return pin_is_high(sw->pin);
}

static __inline__ void SWITCH_init(SWITCH *sw){
	pin_make_input(sw->pin,TRUE);
}

#endif /* SWITCH_H_ */
