/*
 * $Id: timerCapture.c,v 1.2 2009/11/02 19:07:05 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: timerCapture.c,v $
 * Revision 1.2  2009/11/02 19:07:05  clivewebster
 * Added revision log
 *
 * ===========
 *
 * timerCapture.c
 *
 *  Created on: 25-Aug-2009
 *      Author: Clive Webster
 */

#include "timer.h"

void timerCaptureAttach(const Timer* timer, TimerCallback callback, void* user_data, boolean risingEdge )
{
	TimerData* data = timerGetData(timer);
	if(data->capture_callback != null){
		setError(TIMER_CAPTURE_CALLBACK_EXISTS);
	}

	// Turn off interrupts whilst changing
	// in case an interrupt happens half way through
	CRITICAL_SECTION_START;

	// Set callback
	data->capture_callback = callback;
	data->capture_data = user_data;

	// Set the rising falling edge
	if(risingEdge){
		__portMaskSet(&timer->pgm_captureedge);
	}else{
		__portMaskClear(&timer->pgm_captureedge);
	}

	// Clear any pending capture interrupt flag
	timerCaptureClearInterruptPending(timer);

	// enable interrupt
	__portMaskSet(&timer->pgm_captureint);

	CRITICAL_SECTION_END;
}

void timerCaptureDetach(const Timer* timer){
	TimerData* data = timerGetData(timer);

	// stop interrupt
	__portMaskClear(&timer->pgm_captureint);

	// Clear any pending capture interrupt flag
	timerCaptureClearInterruptPending(timer);

	// Set callback
	data->capture_callback = null;
}

