/*
 * $Id: timerCompare.c,v 1.3 2009/11/02 19:07:05 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: timerCompare.c,v $
 * Revision 1.3  2009/11/02 19:07:05  clivewebster
 * Added revision log
 *
 * ===========
 * timerCompare.c
 *
 *  Created on: 25-Aug-2009
 *      Author: Clive Webster
 */
#include "timer.h"

boolean compareIsInUse(const TimerCompare* channel){
	TimerDataCompare* data = compareGetData(channel);

	// if it has a callback then it is in use
	if(data->compare_callback != null)
		return TRUE;

	// if it is channel A and the Timer is is a mode where TOP is set channel A then it is also
	// in use (for defining the Timer TOP value)
	if(timerGetCompare(compareGetTimer(channel),0)==channel){
		TIMER_MODE mode = timerGetMode(compareGetTimer(channel));
		if(mode==TIMER_MODE_CTC_OCR
				|| mode==TIMER_MODE_PWM_PHASE_FREQ_OCR
				|| mode==TIMER_MODE_PWM_PHASE_CORRECT_OCR
				|| mode==TIMER_MODE_PWM_FAST_OCR){
			return TRUE;
		}
	}

	return FALSE;
}

/* -------------------------------------------------------
//
//  Get the compare channel used by a given IOPin
//  May return null if this pin is not to do with a timer
//  or it does not exist on this system board
------------------------------------------------------- */
const TimerCompare* compareFromIOPin(const IOPin* pin){
	uint8_t t;
	uint8_t c;

	// Iterate over the timers
	for(t=0; t < NUMBER_OF_TIMERS; t++){
		const Timer * timer = &pgm_Timers[t];

		// Iterate over the compare units
		uint8_t numCompare = timerNumberOfCompareUnits(timer);
		for(c=0;c<numCompare;c++){
			const TimerCompare* channel = timerGetCompare(timer,c);
			const IOPin* thisPin = compareGetPin(channel);
			if(thisPin != null && thisPin==pin){
				return channel;
			}
		}
	}

	// We cant find the pin
	return null;
}

void nullTimerCompareCallback(const TimerCompare *timer_compare, void* data){}


