/*
 * $Id: timerOverflow.c,v 1.2 2009/11/02 19:07:05 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: timerOverflow.c,v $
 * Revision 1.2  2009/11/02 19:07:05  clivewebster
 * Added revision log
 *
 * ===========
 * timerOverflow.c
 *
 *  Created on: 25-Aug-2009
 *      Author: Clive Webster
 */
#include "timer.h"
/* -------------------------------------------------------
//
//  Attach a callback function to a timer when it overflows
//
//  This function is called during an interrupt so should
//  be quick.
//
------------------------------------------------------- */
void timerOverflowAttach(const Timer* timer, TimerCallback callback, void* user_data )
{
	// Turn off interrupts whilst changing
	// in case an interrupt happens half way through
	CRITICAL_SECTION_START;
	if(callback!=null && timerGetData(timer)->overflow_callback != null){
		setError(TIMER_OVERFLOW_CALLBACK_EXISTS);
	}

	// Set callback
	TimerData* data = timerGetData(timer);
	data->overflow_callback = callback;
	data->overflow_data = user_data;

	// enable interrupt if there is callback
	if(callback!=null){
		// enable overflow interrupts
		__portMaskSet(&timer->pgm_overflowint);
	}else{
		// Disable overflow interrupts
		__portMaskClear(&timer->pgm_overflowint);
	}

	CRITICAL_SECTION_END;
}

