 /* ----------------------------------------------------------------------
 * $Id: uart.h,v 1.9 2009/11/16 03:36:32 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: uart.h,v $
 * Revision 1.9  2009/11/16 03:36:32  clivewebster
 * Fix ATmega8 uart not being included
 *
 * Revision 1.8  2009/11/02 19:07:54  clivewebster
 * Added revision log
 *
 * ===========
 *
 *  Cope with hardware UARTs
 *
 -------------------------------------------------------------------------*/
#ifndef _UART4_H_
#define _UART4_H_

#include "_uart_common.h"

typedef struct s_uart {
	UART				 _uart_;
	const PORT			 statusA;
	const PORT			 statusB;
	const PORT			 baudL;
	const PORT			 baudH;
	const PORT			 data;
	const IOPin*		 spiClock;    // The pin used for SPI clock. eg XCK0 or null if no spi support
	const uint8_t		 u2xMask;
} HW_UART;
extern HW_UART Uarts[];

// Class definition for a hardware UART
extern UART_CLASS c_hw_uart;

// Number of uarts
extern const uint8_t NUM_UARTS;

// Constructor for a buffered Hardware UART
#define MAKE_UART_BUFFERED(rxBuf,txBuf,statusA,statusB,baudL,baudH,data,spi,u2x,rx,tx) \
	{ MAKE_UART_COMMON(rxBuf,txBuf,&c_hw_uart,rx,tx) , \
	_SFR_MEM_ADDR(statusA),_SFR_MEM_ADDR(statusB),_SFR_MEM_ADDR(baudL),_SFR_MEM_ADDR(baudH),_SFR_MEM_ADDR(data),spi,u2x}

// Constructor for a non-buffered Hardware UART
#define MAKE_UART(statusA,statusB,baudL,baudH,data,spi,u2x,rx,tx) \
	MAKE_UART_BUFFERED(null,null,statusA,statusB,baudL,baudH,data,spi,u2x,rx,tx)


//#if !defined(BUILDING_LIBRARY)
// Initialise all of the hardware uarts
// NB This is called automatically at start up
//void __attribute__ ((constructor)) initUarts(void)   {
//	int i;
//	for(i=0;i<NUM_UARTS;i++){
//		uartInit(&(Uarts[i]) ,UART_DEFAULT_BAUD_RATE);
//	}
//}
//#endif


// ----------------- Create some simpler functions for each uart --------------

#if defined(UBRR0) || defined(UBRRL)
MAKE_WRITER(uart0SendByte);
int uart0GetByte(void);
#endif

#ifdef UBRR1
MAKE_WRITER(uart1SendByte);
int uart1GetByte(void);
#endif

#ifdef UBRR2
MAKE_WRITER(uart2SendByte);
int uart2GetByte(void);
#endif

#ifdef UBRR3
MAKE_WRITER(uart3SendByte);
int uart3GetByte(void);
#endif

#endif

