/*
 * $Id: uartsw.h,v 1.4 2009/11/02 19:07:54 clivewebster Exp $
 *
 * Revision History
 * ================
 * $Log: uartsw.h,v $
 * Revision 1.4  2009/11/02 19:07:54  clivewebster
 * Added revision log
 *
 * ===========
 * uartsw.h
 *
 *  Created on: 30-Apr-2009
 *      Author: Clive Webster
 *
 *  This requires a timer which supports input capture
 *  The receive pin must be the Input Capture pin for the timer
 *  The xmit pin can be any IO pin
 *  The first compare channel on the timer is used to schedule transmit bits
 *  The second compare channel on the timer is used to schedule receive bit sampling
 *
 *  Either the receive pin or the transmit can be 'null' if only talking in one direction
 *  If the transmit and receive pins are the same then it uses RS485 half-duplex mode ie the device
 *  talks and then listens.
 *
 *  Device			Timer		Pin
 *  ATMega168		1			B0
 *
 *  ATMega2560		1			D4
 *  				3			E7
 *					4			L0
 *					5			L1
 *
 *  ATMega640		1			D4
 *  				3			E7
 *					4			L0
 *					5			L1
 *
 *  ATMega32		1			D6
 *
 *  ATMega328P		1			B0
 *
 *  ATMega8			1			B0
 */

#ifndef UARTSW_H_
#define UARTSW_H_

#include "_uart_common.h"
#include "iopin.h"

// constants/macros/typdefs
typedef struct s_uart_sw {
	UART				 _uart_;
	const Timer*		timer;		// The 16 bit timer to be used
	volatile uint8_t 	rxData;		// The byte currently being received
	volatile uint8_t 	rxBitNum;	// The bit number currently being received
	volatile uint8_t 	txData;		// The byte currently being transmitted
	volatile uint8_t 	txBitNum;	// The bit number currently being transmitted
	volatile uint16_t 	baudRateDiv;// The time between bits
	boolean				inverted;	// Is it inverted
} SW_UART;

// Class definition for a software UART
extern UART_CLASS c_sw_uart;

// Constructor for a buffered software UART
#define MAKE_SW_UART_BUFFERED(rxBuf,txBuf,timer,rxpin,txpin,inv) \
	{ MAKE_UART_COMMON(rxBuf,txBuf,&c_sw_uart,rxpin,txpin) , \
	timer,0,0,0,0,0,inv }


// functions

#endif /* UARTSW_H_ */
