/*
 * version.h
 *
 *      Author: Clive Webster
 *
 *      Version 1.0 Beta release 2 April 2009
 *
 *      Version 1.1 Beta release 8 May 2009
 *      	Fixed issuse with interrupts becoming disabled in buffer.c
 *      	Added support for software UARTs and GPS
 *
 *      Version 1.2 Beta release 2 June 2009
 *      	Fixed compile problems with serial motor drivers
 *      	Added alpha version for AxonII support via 'sys/axon2.h'
 *      	Added constant in timer.h for 'NUMBER_OF_TIMERS'
 *      	Added function in timer.h 'const IOPin* timerGetCapturePin(const Timer* timer)' to return the input capture pin
 *      		for the specified timer or null if it doesn't have one
 *      	Replaced appInit with calls to appInitHardware and appInitSoftware
 *			appControl can now return a value in us.
 *
 *		Version 1.3 Beta release 20 June 2009
 *			Fixed sabertooth issues and added choice of PACKETIZED or SIMPLE.
 *			Stopped automatic initialisation of UARTs as this caused random glitch characters to be sent
 *				at power up. Serial motor drivers still do it for you but you will now need to init any
 *				uart used by rprintf statements
 *
 *		Version 1.4
 *			Added ATMega2560 support
 *			Added 'CLAMP' to 'libdefs.h'
 *			Changed the signature of the code for rprintfInit to return the value to make it easier to do checksums etc
 *			Added SPI support in hardware (spi.h), software (spisw.h), and uart (spiUart.h)
 *			Added EEPROM support
 *
 *		Version 1.5
 *			Fixed a2d to read from multiple ports - wasn't checking for 'read complete' correctly so gave old values
 *			Fixed servos.h so that rather than calling 20ms since previous bank start it now uses 20ms since the last
 *				servo of the same bank start. So delay may well be > 20ms per servo
 *			Added example files
 *			rprintf now supports %s for printing a string from RAM
 *			rprintf.h now has void rprintfCharN(unsigned char c, size_t repeat); to print a char a number of times
 *			rprintf.h now has void rprintfMemoryDump(const void* data, size_t offset, size_t len); to dump a tabular view
 *			Added 'color.h'
 *			Added 'fraction.h' - not documented as you need to understand Fibonacci sequences. Too long to describe !! But is used
 *				by other lib functions to avoid floating point maths library.
 *
 *		Version 1.6
 *			Added I2C master mode support via i2c_master.h
 *			Changed the implementation of the delay functions, scheduler, and servos.h to give more
 *					accurate timings and less servo jitter. These changes have NO EFFECT on your own program.
 *			Added initial support for the Blackfin camera from Surveyor Corporation
 *
 *		Version 1.7
 *			Fixed issues in I2C master mode, pullups now turned on
 *			Added Motors/Toshiba/TB6612FNG.h motor driver
 *			iopin.h Added pulseIn and pulseOut
 *			sensors - restructured it all, added HMC6343 compass, ACCM3D2 accelerometer
 *			Removed original sensor.h that read a single value as some sensors can have many values.
 *			Sensors are now grouped by 'type', 'manufacturer' and 'device'.
 *			All sensors of the same 'type' will return values in the same range so that they are
 *			interchangeable. All sensors should now be initialised in your appInitHardware. These calls
 *			are of the format: <type>Init(sensor) and can be read using <type>Read(sensor) where
 *			<type> is the type of sensor eg: distance, compass, voltage etc.
 *			The <type>Read function will access the device and store the result locally. This value can
 *			be retrieved by reading: sensor.<type>.<unit>
 *			EG distance sensors return a value in cm. So the value will be in sensor.distance.cm
 *			The manual has been updated with these changes
 *
 *		Version 1.8
 *			Added a routine for each sensor type to dump its values to the rprintf destination.
 *			Added MPX5100a pressure sensor - untested
 *			Added ADXL335 accelerometer from Analog Devices - untested
 *			Fixed ACCM3D2 sensor - tested and okayed by Admin
 *			Fixed HMC6343 sensor - tested and okayed by Admin
 *			Added Gyro sensor IDG500 - untested
 *
 *		Version 1.9  1 October 2009
 *			In servos.h added servosSetConnected and servosSetSpeed to work in groups of servos
 *			Added Sensors/Gyro/InvenSense/IDG300.h
 *			Added Sensors/Gyro/ST/LPR530AL.h
 *			Added Sensors/Gyro/ST/LY530ALH.h
 *			Added support for ATMega328p (32k flash, 2k RAM, 1k EEPROM in 28 pin DIP - compatible with ATMega168)
 *			Fixed timer glitch
 *			Changed gps so that baud rate should be set by the user rather than forcing 4800 baud
 *
 *		Version 1.10 15 October 2009
 *			Accelerometers are so flakey! Use existing code to find min/max values for each axis then
 *				to make them more precise try adding
 *				accelerometerCalibrateX(device,min,max)
 *				accelerometerCalibrateY(device,min,max)
 *				accelerometerCalibrateZ(device,min,max)
 *				in your appInitHardware to force the sensor to return sensible values.
 *			Tested and fixed all sonars MaxBotix EZ1, Parallax Ping sonar, Devantech SRF-05
 *			Tested and fixed BabyOrangutan B328 board. On board motors working.
 *			Sabertooth.h moved from Motors/Dimension to Motors/DimensionEngineering
 *			Added supprt for LB1836M as used by older BabyOrangutan boards
 *			Changed Toshiba TB6612FNG to allow either 3 or 2 pin control
 *			Fixed conversions for Sharp IR distance sensors
 *			Each sys file now has a sysInitHardware to init any devices built onto the board
 *			All versions of rprintf now allow %u to print unsigned number
 *			Removed 'servoPWM.h' - its only function is now merged into 'servos.h'
 *			timer.h - added 'compareGetOutputMode' function
 *
 *		Version 1.11 27 October 2009
 *			Added pwm.h to allow you set up a hardware PWM of your choosing - see pwm.h
 *			Added interpolateU to core.h - same as 'interpolate' but copes better if the returned value
 *				is always an unsigned (positive) number
 *
 *		Version 1.12 3 November 2009
 *			Added Freescale MMA7260QT accelerometer
 *			Fixed ATmega328B UART functions
 *			Fixed Documentation example for Maxbotix EZ1
 *			Fixed some case sensitive file names for Unix
 *
 *	    Version 1.13 28 December 2009
 *	    	Fixed #includes within library for Unix (as best I can)
 *	    	Add Sensors/Distance/Devantech/SRF04_Sonar.h
 *	    	Changed blackfin camera initialisation. Old way was a bit more clumsy ie
 *	    		MAKE_BLACKFIN_CAMERA(&(UART3->__uart__)) but is now simpler ie MAKE_BLACKFIN_CAMERA(UART3);
 *			Changed blackfinFetchBlob so that the first blob# starts at 0 rather than 1
 *			Add ledseg.h to drive 8 segment leds, and also to create a marquee
 *			Change Axon2.h to set default rprintf and error log locations to go to the
 *				built in 8 segment led.
 *			Changed Axon2.h - renamed led_put_digit to led_put_char as it can now cope with more than
 *				just digits
 *		Version 1.13c 17 January 2010
 *				gps now returns track - see gps.h
 *				gpsNMEA - turned off debug mode
 *		Version 1.14 5 February 2010
 *			Changed handling of ADC pins. No longer assumes that ADC inputs are IOPins. This means that any sensors whose MAKE command
 *				specified an IOPin for an analogue input now specify an ADC_CHANNEL instead ie ADC0, ADC1, etc.
 *			Removed functions a2dConvert10bitIO and a2dConvert8bitIO as ADCs no longer have to be IOPins
 *			Added uartSendBuffer to send a sequence of bytes over the uart
 *			Added switch.h to wrap an IOPin with some more friendly macro calls for a switch.
 *			Changed Marquee to introduce a brief 'flash' so that if two chars are the same then there is a brief blink between them
 *			On Axon II removed the individual LED definitions for the segment display. They have been
 *				replaced with a single SEGLED called 'led_display'. So to turn on segment E then use
 *				segled_on(&led_display, SEGMENT_E); // turn on  segment E
 *				segled_off(&led_display,SEGMENT_E); // turn off segment E
 *			Fixed ATMega32 timer0 compare problem in device.h
 *			Added HMC6352 compass - but untested as of yet. If you use it then let me know.
 *			Added some functions to HMC6343 to allow change of reset frequency
 *			Replaced all the Sharp distance sensor header files with one file called GP2.h
 *			Added Sharp GP2Y0A02YK0F, GP2Y0A700K0F and GP2Y0A710K0F distance sensors
 *			Added Sharp GP2Y3A002K0F, GP2Y3A003K0F 'wide' sensors
 *			Moved gps.h to Sensors/GPS/gps.h
 *			Moved gpsNMEA.h to Sensors/GPS/NMEA/gpsNMEA.h
 *			MAKE_GPS_NMEA has an extra parameter for the baud rate. You no longer need to initialise
 *				the UART yourself.It just happens. This was done to reduce buffer overflows at powerup.
 *			Added alpha support for AVRcam - can't test it - don't have one and they aren't available!
 *			Stop the sys files from making pins into outputs in 'configure ports'. Its "mega-bad" as it can cause temporary short circuits and,
 *				as found/fixed earlier, can really upset UARTs.
 *		Version 1.14a 6 February 2010
 *			Fixed bug in flashing characters in the Marquee
 *			Added servoSetConfig in servos.h to allow change of servo config at runtimer
 */


#ifndef VERSION_H_
#define VERSION_H_


#endif /* VERSION_H_ */
